library c3_5_routetransition;

import 'package:flutter/material.dart';

enum RouteTransitionAnimation { normal, fade, slide, scale, rotation, size }

/// Main class. [context] is the BuildContext application at the moment,
/// [child] is the Widget to navigate, [animation] is the animation type.
class RouteTransition {
  final BuildContext context;
  final Widget child;
  final Curve curve;
  final RouteTransitionAnimation animation;
  final Duration duration;
  final double from;
  final bool replacement;

  late Route _route;

  RouteTransition({
    required this.context,
    required this.child,
    this.curve = Curves.easeInOut,
    this.animation = RouteTransitionAnimation.normal,
    this.duration = const Duration(milliseconds: 300),
    this.from = 0.0,
    this.replacement = false,
  }) {
    switch (animation) {
      case RouteTransitionAnimation.normal:
        _route = _normalTransition();
        break;
      default:
        _route = _transition();
    }

    (replacement) ? _pushReplacementPage() : _pushPage();
  }

  /// Normal transition code
  Route _normalTransition() => MaterialPageRoute(builder: (_) => child);

  /// Generalization of the needed PageRouteBuilder, and a transition controller fuction.
  Route _transition() => PageRouteBuilder(
      pageBuilder: (_, __, ___) => child,
      transitionDuration: duration,
      transitionsBuilder: (_, animation, __, child) {
        switch (this.animation) {
          case RouteTransitionAnimation.fade:
            return _fadeTransition(animation);
          case RouteTransitionAnimation.slide:
            return _slideTransition(animation);
          case RouteTransitionAnimation.scale:
            return _scaleTransition(animation);
          case RouteTransitionAnimation.rotation:
            return _rotationTransition(animation);
          case RouteTransitionAnimation.size:
            return _sizeTransition(animation);
          default:
            return _fadeTransition(animation);
        }
      });

  /// FadeTransition function.
  Widget _fadeTransition(Animation<double> animation) => FadeTransition(
        child: child,
        opacity: Tween<double>(begin: from, end: 1.0)
            .animate(CurvedAnimation(parent: animation, curve: curve)),
      );

  /// SlideTransition function.
  Widget _slideTransition(Animation<double> animation) => SlideTransition(
        child: child,
        position: Tween<Offset>(begin: const Offset(0.0, 1.0), end: Offset.zero)
            .animate(CurvedAnimation(parent: animation, curve: curve)),
      );

  /// ScaleTransition function.
  Widget _scaleTransition(Animation<double> animation) => ScaleTransition(
        child: child,
        scale: Tween<double>(begin: 0.0, end: 1.0)
            .animate(CurvedAnimation(parent: animation, curve: curve)),
      );

  /// RotationTransition function.
  Widget _rotationTransition(Animation<double> animation) => RotationTransition(
        child: child,
        turns: Tween<double>(begin: from, end: 1.0)
            .animate(CurvedAnimation(parent: animation, curve: curve)),
      );

  /// SizeTransition function.
  Widget _sizeTransition(Animation<double> animation) => SizeTransition(
        child: child,
        sizeFactor: Tween<double>(begin: from, end: 1.0)
            .animate(CurvedAnimation(parent: animation, curve: curve)),
      );

  /// Push to the Page
  void _pushPage() => Navigator.push(context, _route);

  /// PushReplacement to the Page
  void _pushReplacementPage() => Navigator.pushReplacement(context, _route);
}
